print("import: 'cpnest'")
import cpnest

print("import: 'cpnest.NestedSampling'")
import cpnest.NestedSampling

print("import: 'cpnest.cpnest'")
import cpnest.cpnest

print("import: 'cpnest.model'")
import cpnest.model

print("import: 'cpnest.nest2pos'")
import cpnest.nest2pos

print("import: 'cpnest.parameter'")
import cpnest.parameter

print("import: 'cpnest.proposal'")
import cpnest.proposal

print("import: 'cpnest.sampler'")
import cpnest.sampler

print("import: 'cpnest.logger'")
import cpnest.logger

